--[[
    Aaron MacDonald
    August 28, 2010
]]

require('graphics')
require('constants')
require('utility')
require('vector')

local SPRITE_NORTH = '_n'
local SPRITE_EAST = '_e'
local SPRITE_SOUTH = '_s'
local SPRITE_WEST = '_w'

actor = {}

function actor.diagonalSpeed(speed)
    return (speed * speed) / math.sqrt((speed * speed) * 2)
end

function actor.newActor(world, actorType, x, y, facing)
    local body = love.physics.newBody(world, x, y, 0, 0)
    local shape = love.physics.newCircleShape(body, 0, 0,
      actorType.radius)
    shape:setRestitution(0)
    shape:setData(constants.ACTOR)

    body:setMassFromShapes()
    body:setBullet(true) -- I'm a sucker for
                         -- continuous collision detection

    local spriteSheet = graphics.loadImage(actorType.spriteSheet)
    local sheetWidth = spriteSheet:getWidth()
    local sheetHeight = spriteSheet:getHeight()

    local actWidth = actorType.spriteWidth
    local actHeight = actorType.spriteHeight

    local spriteQuads = {
      [constants.UP] = love.graphics.newQuad(0, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.RIGHT] = love.graphics.newQuad( actWidth, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.DOWN] = love.graphics.newQuad(actWidth * 2, 0,
        actWidth, actHeight, sheetWidth, sheetHeight),
      [constants.LEFT] = love.graphics.newQuad(actWidth * 3, 0,
        actWidth, actHeight, sheetWidth, sheetHeight)
    }

    local act = {
        type = actorType,

        body = body,
        shape = shape,

        spriteSheet = spriteSheet,
        spriteQuads = spriteQuads,

        facing = facing
    }

    return act
end

function actor.getX(act)
    return act.body:getX()
end

function actor.getY(act)
    return act.body:getY()
end

function actor.getFloorX(act)
    return math.floor(actor.getX(act))
end

function actor.getFloorY(act)
    return math.floor(actor.getY(act))
end

function actor.controlMovement(act, movementControls)
    local velocity = vect.newVector()

    local speed = act.type.speed
    local diagonalSpeed = act.type.diagonalSpeed

    -- Velocity

    if movementControls[constants.RIGHT] then
        velocity.x = speed
    end
    if movementControls[constants.LEFT] then
        velocity.x = velocity.x - speed
    end

    if movementControls[constants.UP] then
        velocity.y = -speed
    end
    if movementControls[constants.DOWN] then
        velocity.y = velocity.y + speed
    end

    -- Diagonal velocity

    if (velocity.x ~= 0) and (act.dy ~= 0) then
        velocity.x = diagonalSpeed * utility.sign(velocity.x)
        velocity.y = diagonalSpeed * utility.sign(velocity.y)
    end

    -- Facing

    if velocity.x == 0 then
        if velocity.y > 0 then
            act.facing = constants.DOWN
        elseif velocity.y < 0 then
            act.facing = constants.UP
        end
    elseif velocity.y == 0 then
        if velocity.x > 0 then
            act.facing = constants.RIGHT
        elseif velocity.x < 0 then
            act.facing = constants.LEFT
        end
    end

    act.body:wakeUp()
    act.body:setLinearVelocity(velocity.x, velocity.y)
end

function actor.draw(act)
    local x = actor.getFloorX(act)
    local y = actor.getFloorY(act)

    love.graphics.setColor(255, 255, 255, 128)
    love.graphics.circle('fill', x, y, act.type.radius, 32)

    love.graphics.setColor(255, 255, 255)
    love.graphics.drawq(act.spriteSheet, act.spriteQuads[act.facing],
      x - act.type.spriteOffsetX, y - act.type.spriteOffsetY)
end