--[[
    Aaron MacDonald
    August 28, 2010
]]

require('utility')

vect = {}

function vect.newVector(x, y)
    return {x = x or 0, y = y or 0}
end

function vect.isZero(v)
    return (v.x == 0) and (v.y == 0)
end

function vect.floor(v)
    local x = utility.floor2(v.x)
    local y = utility.floor2(v.y)
    return vect.newVector(x, y)
end

function vect.add(va, vb)
    return vect.newVector(va.x + vb.x, va.y + vb.y)
end

function vect.subtract(va, vb)
    return vect.newVector(va.x - vb.x, va.y - vb.y)
end

function vect.scale(v, s)
    return vect.newVector(v.x * s, v.y * s)
end

function vect.magnitudeSqr(v)
    return (v.x * v.x) + (v.y * v.y)
end

function vect.magnitude(v)
    return math.sqrt(vect.magnitudeSqr(v))
end

function vect.normal(v)
    assert(not vect.isZero(v))

    local m = vect.magnitude(v)
    local nx = v.x / m
    local ny = v.y / m

    return vect.newVector(nx, ny)
end

function vect.dotProduct(va, vb)
    return (va.x * vb.x) + (va.y * vb.y)
end

function vect.projection(va, vb)
    -- Project va onto vb
    local dp = vect.dotProduct(va, vb)
    local mbSqr = vect.magnitudeSqr(vb)

    local px = (dp / mbSqr) * vb.x
    local py = (dp / mbSqr) * vb.y

    return vect.newVector(px, py)
end

function vect.rightHandNormal(v)
    return vect.newVector(-v.y, v.x)
end

function vect.leftHandNormal(v)
    return vect.newVector(v.y, -v.x)
end